/*
 * Copyright (C) 2013 Jolla Ltd. <chris.adams@jollamobile.com>
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of Nemo Mobile nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 */

#ifndef FACEBOOKCOMMENTINTERFACE_H
#define FACEBOOKCOMMENTINTERFACE_H

#include "identifiablecontentiteminterface.h"

#include "facebookobjectreferenceinterface.h"
#include <QtCore/QString>

/*
 * NOTE: if you construct one of these in C++ directly,
 * you MUST call classBegin() and componentCompleted()
 * directly after construction.
 */

class FacebookCommentInterfacePrivate;
class FacebookCommentInterface: public IdentifiableContentItemInterface
{
    Q_OBJECT
    Q_PROPERTY(FacebookObjectReferenceInterface * from READ from NOTIFY fromChanged)
    Q_PROPERTY(QString message READ message NOTIFY messageChanged)
    Q_PROPERTY(QString createdTime READ createdTime NOTIFY createdTimeChanged)
    Q_PROPERTY(int likeCount READ likeCount NOTIFY likeCountChanged)
    Q_PROPERTY(FacebookCommentInterface * parent READ parent NOTIFY parentChanged)
    Q_PROPERTY(bool canComment READ canComment NOTIFY canCommentChanged)
    Q_PROPERTY(int commentCount READ commentCount NOTIFY commentCountChanged)
    Q_PROPERTY(bool liked READ liked NOTIFY likedChanged)
public:
    explicit FacebookCommentInterface(QObject *parent = 0);

    // Overrides.
    int type() const;
    Q_INVOKABLE bool remove();
    Q_INVOKABLE bool reload(const QStringList &whichFields = QStringList());

    // Invokable API.
    Q_INVOKABLE bool like();
    Q_INVOKABLE bool unlike();

    // Accessors
    FacebookObjectReferenceInterface * from() const;
    QString message() const;
    QString createdTime() const;
    int likeCount() const;
    FacebookCommentInterface * parent() const;
    bool canComment() const;
    int commentCount() const;
    bool liked() const;
Q_SIGNALS:
    void fromChanged();
    void messageChanged();
    void createdTimeChanged();
    void likeCountChanged();
    void parentChanged();
    void canCommentChanged();
    void commentCountChanged();
    void likedChanged();
private:
    Q_DECLARE_PRIVATE(FacebookCommentInterface)
};

#endif // FACEBOOKCOMMENTINTERFACE_H
